@@ -2,9 +2,9 @@
 
 Copyright (c) 1999-2004 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: BIO.py 299 2005-06-09 17:32:28Z heikki $'
+import m2 
 
-import m2
+# Deprecated
 from m2 import bio_do_handshake as bio_do_ssl_handshake
 
 from cStringIO import StringIO
@@ -17,16 +17,18 @@ class BIO:
 
     """Abstract object interface to the BIO API."""
 
+    m2_bio_free = m2.bio_free
+
     def __init__(self, bio=None, _pyfree=0, _close_cb=None):
         self.bio = bio
         self._pyfree = _pyfree
         self._close_cb = _close_cb
         self.closed = 0
         self.write_closed = 0
-
+        
     def __del__(self):
         if self._pyfree:
-            m2.bio_free(self.bio)
+            self.m2_bio_free(self.bio)
 
     def _ptr(self):
         return self.bio
@@ -89,14 +91,38 @@ class BIO:
         m2.bio_flush(self.bio)
 
     def reset(self):
-        raise NotImplementedError
+        """
+        Sets the bio to its initial state
+        """
+        return m2.bio_reset(self.bio)
 
     def close(self):
         self.closed = 1
         if self._close_cb:
             self._close_cb()
 
+    def should_retry(self):
+        """
+        Can the call be attempted again, or was there an error
+        ie do_handshake 
+       
+        """
+        return m2.bio_should_retry(self.bio) 
 
+    def should_read(self):
+        """
+        Returns whether the cause of the condition is the bio
+        should read more data
+        """
+        return m2.bio_should_read(self.bio)
+    
+    def should_write(self):
+        """
+        Returns whether the cause of the condition is the bio
+        should write more data
+        """
+        return m2.bio_should_write(self.bio)
+    
 class MemoryBuffer(BIO):
 
     """
@@ -169,6 +195,9 @@ class IOBuffer(BIO):
     that makefile() of said underlying SSL socket works.
     """
 
+    m2_bio_pop = m2.bio_pop
+    m2_bio_free = m2.bio_free
+
     def __init__(self, under_bio, mode='rwb', _pyfree=1):
         BIO.__init__(self, _pyfree=_pyfree)
         self.io = m2.bio_new(m2.bio_f_buffer())
@@ -179,11 +208,11 @@ class IOBuffer(BIO):
             self.write_closed = 0
         else:
             self.write_closed = 1
-
+            
     def __del__(self):
-        if self._pyfree:
-            m2.bio_pop(self.bio)
-            m2.bio_free(self.io)
+        if getattr(self, '_pyfree', 0):
+            self.m2_bio_pop(self.bio)
+            self.m2_bio_free(self.io)
 
     def close(self):
         BIO.close(self)
@@ -197,29 +226,59 @@ class CipherStream(BIO):
 
     SALT_LEN = m2.PKCS5_SALT_LEN
 
+    m2_bio_pop = m2.bio_pop
+    m2_bio_free = m2.bio_free
+
     def __init__(self, obio):
         BIO.__init__(self, _pyfree=1)
         self.obio = obio
         self.bio = m2.bio_new(m2.bio_f_cipher())
         self.closed = 0
-
+        
     def __del__(self):
-        if not self.closed:
+        if not getattr(self, 'closed', 1):
             self.close()
 
     def close(self):
-        m2.bio_pop(self.bio)
-        m2.bio_free(self.bio)
+        self.m2_bio_pop(self.bio)
+        self.m2_bio_free(self.bio)
         self.closed = 1
         
     def write_close(self):
         self.obio.write_close()
 
     def set_cipher(self, algo, key, iv, op):
-        cipher = getattr(m2, algo)
-        if not cipher:
+        cipher = getattr(m2, algo, None)
+        if cipher is None:
             raise ValueError, ('unknown cipher', algo)
         m2.bio_set_cipher(self.bio, cipher(), key, iv, op) 
         m2.bio_push(self.bio, self.obio._ptr())
 
 
+class SSLBio(BIO):
+    """
+    Object interface to BIO_f_ssl 
+    """
+    def __init__(self, _pyfree=1):
+        BIO.__init__(self, _pyfree)
+        self.bio = m2.bio_new(m2.bio_f_ssl())
+        self.closed = 0
+
+   
+    def set_ssl(self, conn, close_flag=m2.bio_noclose):
+        """
+        Sets the bio to the SSL pointer which is
+        contained in the connection object.  
+        """
+        self._pyfree = 0 
+        m2.bio_set_ssl(self.bio, conn.ssl, close_flag)
+       
+    def do_handshake(self):
+        """
+        Do the handshake.
+        
+        Return 1 if the handshake completes
+        Return 0 or a negative number if there is a problem
+        """
+        return m2.bio_do_handshake(self.bio)
+
