@@ -2,8 +2,6 @@
 
 Copyright (c) 1999-2004 Ng Pheng Siong. All rights reserved."""
 
-RCS_id = '$Id: Connection.py 291 2005-06-09 02:50:13Z heikki $'
-
 # Python
 import socket
 
@@ -17,7 +15,7 @@ import Checker
 #SSLError = getattr(__import__('M2Crypto.SSL', globals(), locals(), 'SSLError'), 'SSLError')
 from M2Crypto.SSL import SSLError
 
-def _serverPostConnectionCheck(peerX509, expectedHost):
+def _serverPostConnectionCheck(*args, **kw):
     return 1
 
 class Connection:
@@ -27,6 +25,8 @@ class Connection:
     clientPostConnectionCheck = Checker.Checker()
     serverPostConnectionCheck = _serverPostConnectionCheck
 
+    m2_bio_free = m2.bio_free
+
     def __init__(self, ctx, sock=None):
         self.ctx = ctx
         self.ssl = m2.ssl_new(self.ctx.ctx)
@@ -36,13 +36,14 @@ class Connection:
             self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self._fileno = self.socket.fileno()
-
+        
+        self.blocking = self.socket.gettimeout()
+        
     def __del__(self):
-        try:
-            m2.bio_free(self.sslbio)
-            m2.bio_free(self.sockbio)
-        except AttributeError:
-            pass
+        if getattr(self, 'sslbio', None):
+            self.m2_bio_free(self.sslbio)
+        if getattr(self, 'sockbio', None):
+            self.m2_bio_free(self.sockbio)
         self.socket.close()
 
     def close(self):
@@ -71,10 +72,32 @@ class Connection:
     def ssl_get_error(self, ret):
         return m2.ssl_get_error(self.ssl, ret)
 
+    def set_bio(self, readbio, writebio):
+        """
+        Explicitly set read and write bios
+        """
+        m2.ssl_set_bio(self.ssl, readbio._ptr(), writebio._ptr())
+        
     def set_client_CA_list_from_file(self, cafile):
-        m2.ssl_set_client_CA_list(self.ssl, cafile)
+        """
+        Set the acceptable client CA list. If the client
+        returns a certificate, it must have been issued by
+        one of the CAs listed in cafile.
+        
+        Makes sense only for servers.
+        
+        @param cafile: Filename from which to load the CA list.
+        """
+        m2.ssl_set_client_CA_list_from_file(self.ssl, cafile)
 
     def set_client_CA_list_from_context(self):
+        """
+        Set the acceptable client CA list. If the client
+        returns a certificate, it must have been issued by
+        one of the CAs listed in context.
+        
+        Makes sense only for servers.
+        """
         m2.ssl_set_client_CA_list_from_context(self.ssl, self.ctx.ctx)
 
     def setup_addr(self, addr):
@@ -111,9 +134,9 @@ class Connection:
         ssl.setup_ssl()
         ssl.set_accept_state()
         ssl.accept_ssl()
-        check = getattr(self, 'postConnectionCheck', Connection.serverPostConnectionCheck)
+        check = getattr(self, 'postConnectionCheck', self.serverPostConnectionCheck)
         if check is not None:
-            if not check(self.get_peer_cert(), self.addr[0]):
+            if not check(self.get_peer_cert(), ssl.addr[0]):
                 raise Checker.SSLVerificationError, 'post connection check failed'
         return ssl, addr
 
@@ -129,7 +152,7 @@ class Connection:
         self.setup_ssl()
         self.set_connect_state()
         ret = self.connect_ssl()
-        check = getattr(self, 'postConnectionCheck', Connection.clientPostConnectionCheck)
+        check = getattr(self, 'postConnectionCheck', self.clientPostConnectionCheck)
         if check is not None:
             if not check(self.get_peer_cert(), self.addr[0]):
                 raise Checker.SSLVerificationError, 'post connection check failed'
@@ -163,18 +186,22 @@ class Connection:
             raise ValueError, 'size <= 0'
         return m2.ssl_read_nbio(self.ssl, size)
 
-    sendall = send = write = _write_bio
-    recv = read  = _read_bio
+    def write(self, data):
+        if self.blocking:
+            return self._write_bio(data)
+        return self._write_nbio(data)
+    sendall = send = write
+    
+    def read(self, size=1024):
+        if self.blocking:
+            return self._read_bio(size)
+        return self._read_nbio(size)
+    recv = read
 
     def setblocking(self, mode):
         """Set this connection's underlying socket to _mode_."""
         self.socket.setblocking(mode)
-        if mode:
-            self.send = self.write = self._write_bio
-            self.recv = self.read = self._read_bio
-        else:
-            self.send = self.write = self._write_nbio
-            self.recv = self.read = self._read_nbio
+        self.blocking = mode
 
     def fileno(self):
         return self.socket.fileno()
@@ -260,7 +287,7 @@ class Connection:
         m2mode = ['', 'r'][r] + ['', 'w'][w] + ['', 'b'][b]      
         # XXX Need to dup().
         bio = BIO.BIO(self.sslbio, _close_cb=self.close)
-        BIO.bio_do_ssl_handshake(bio._ptr())
+        m2.bio_do_handshake(bio._ptr())
         return BIO.IOBuffer(bio, m2mode)
 
     def getsockname(self):
