@@ -4,8 +4,6 @@
 
 Copyright (c) 1999-2002 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: test_bio_file.py 299 2005-06-09 17:32:28Z heikki $'
-
 import unittest
 import M2Crypto
 from M2Crypto.BIO import File, openfile
@@ -15,9 +13,9 @@ class FileTestCase(unittest.TestCase):
 
     def setUp(self):
         self.data = 'abcdef' * 64
-        if sys.version[0] == '2' and sys.platform != 'win32':
+        if sys.platform != 'win32':
             self.fname = os.tmpnam()
-        elif sys.version[:3] == '1.5' or sys.platform == 'win32':
+        else:
             import tempfile
             self.fname = tempfile.mktemp()
 
