@@ -4,42 +4,42 @@
 
 Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""
 
-RCS_id = '$Id: test_dsa.py 299 2005-06-09 17:32:28Z heikki $'
-
 import unittest
 import sha
 from M2Crypto import DSA, BIO, Rand, m2
 
 class DSATestCase(unittest.TestCase):
 
-    errkey = 'rsa.priv.pem'
+    errkey  = 'rsa.priv.pem'
     privkey = 'dsa.priv.pem'
-    param = 'dsa.param.pem'
+    pubkey  = 'dsa.pub.pem'
+    param   = 'dsa.param.pem'
 
     data = sha.sha('Can you spell subliminal channel?').digest()
+    different_data = sha.sha('I can spell.').digest()
 
     def callback(self, *args):
         pass
 
-    def callback2(self):
-        pass
-
     def check_loadkey_junk(self):
-        self.assertRaises(ValueError, DSA.load_key, self.errkey)
+        self.assertRaises(DSA.DSAError, DSA.load_key, self.errkey)
 
     def check_loadkey(self):
         dsa = DSA.load_key(self.privkey)
         assert len(dsa) == 512
 
     def check_loadparam(self):
-        # XXX more work needed
+        self.assertRaises(DSA.DSAError, DSA.load_key, self.param)
         dsa = DSA.load_params(self.param)
+        assert not dsa.check_key()
         assert len(dsa) == 512
 
     def check_sign(self):
         dsa = DSA.load_key(self.privkey)
+        assert dsa.check_key()
         r, s = dsa.sign(self.data)
         assert dsa.verify(self.data, r, s)
+        assert not dsa.verify(self.data, s, r)
 
     def check_sign_asn1(self):
         dsa = DSA.load_key(self.privkey)
@@ -51,20 +51,37 @@ class DSATestCase(unittest.TestCase):
         self.assertRaises(AssertionError, dsa.sign, self.data)
         self.assertRaises(AssertionError, dsa.sign_asn1, self.data)
 
-    def check_verify(self):
+    def check_pub_verify(self):
         dsa = DSA.load_key(self.privkey)
         r, s = dsa.sign(self.data)
+        dsapub = DSA.load_pub_key(self.pubkey)
+        assert dsapub.check_key()
+        assert dsapub.verify(self.data, r, s)
+
+    def check_verify_fail(self):
+        dsa = DSA.load_key(self.privkey)
+        r, s = dsa.sign(self.data)
+        assert not dsa.verify(self.different_data, r, s)
+
+    def check_verify_fail2(self):
+        dsa = DSA.load_key(self.privkey)
+        r,s = dsa.sign(self.data)
         dsa2 = DSA.load_params(self.param)
+        assert not dsa2.check_key()
         self.assertRaises(AssertionError, dsa2.verify, self.data, r, s)
 
-    def check_genparam(self):
+    def check_genparam_setparam_genkey(self):
         dsa = DSA.gen_params(256, self.callback)
         assert len(dsa) == 512
-
-    def check_genparam_bad_cb(self):
-        dsa = DSA.gen_params(256, self.callback2)
-        assert len(dsa) == 512
-
+        p = dsa.p
+        q = dsa.q
+        g = dsa.g
+        dsa2 = DSA.set_params(p,q,g)
+        assert not dsa2.check_key()
+        dsa2.gen_key()
+        assert dsa2.check_key()
+        r,s = dsa2.sign(self.data)
+        assert dsa2.verify(self.data, r, s)
 
 def suite():
     return unittest.makeSuite(DSATestCase, 'check')
