@@ -4,7 +4,18 @@
 
 Copyright (c) 2000-2004 Ng Pheng Siong. All rights reserved."""
 
-RCS_id='$Id: test_ssl.py 310 2005-07-29 23:23:50Z heikki $'
+"""
+TODO
+
+Server tests:
+- ???
+
+Others:
+- ssl_dispatcher
+- SSLServer
+- ForkingSSLServer
+- ThreadingSSLServer
+"""
 
 import os, socket, string, sys, tempfile, thread, time, unittest
 from M2Crypto import Rand, SSL, m2
@@ -16,7 +27,8 @@ def verify_cb_new_function(ok, store):
     try:
         assert not ok
         err = store.get_error()
-        assert err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
+        assert err == m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT or \
+               err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
                err == m2.X509_V_ERR_CERT_UNTRUSTED or \
                err == m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE
         app_data = m2.x509_store_ctx_get_app_data(store.ctx)
@@ -64,7 +76,9 @@ class SSLClientTestCase(unittest.TestCase):
         self.srv_port = srv_port
         self.srv_addr = (srv_host, srv_port)
         self.srv_url = 'https://%s:%s/' % (srv_host, srv_port)
-        self.args = ['s_server', '-quiet', '-www', '-accept', str(self.srv_port)]
+        self.args = ['s_server', '-quiet', '-www',
+                     #'-cert', 'server.pem', Implicitly using this
+                     '-accept', str(self.srv_port)]
 
     def tearDown(self):
         global srv_port
@@ -73,6 +87,7 @@ class SSLClientTestCase(unittest.TestCase):
     def test_server_simple(self):
         pid = self.start_server(self.args)
         try:
+            self.assertRaises(ValueError, SSL.Context, 'tlsv5')
             ctx = SSL.Context()
             s = SSL.Connection(ctx)
             s.connect(self.srv_addr)
@@ -127,14 +142,7 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             ctx = SSL.Context('sslv23')
             s = SSL.Connection(ctx)
-            try:
-                s.connect(self.srv_addr)
-            except SSL.SSLError, e:
-                # XXX Not totally sure if this is correct, but SSLv2 handling DID change.
-                if m2.OPENSSL_VERSION_NUMBER < 0x90800F:
-                    self.failUnlessEqual(e[0], 'unsupported protocol')
-                else:
-                    self.failUnlessEqual(e[1], 'Software caused connection abort')
+            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
             s.close()
         finally:
             self.stop_server(pid)
@@ -217,6 +225,22 @@ class SSLClientTestCase(unittest.TestCase):
             s.set_cipher_list('EXP-RC4-MD5')
             s.connect(self.srv_addr)
             data = self.http_get(s)
+            
+            cipher_stack = s.get_ciphers()
+            assert cipher_stack[0].name() == 'EXP-RC4-MD5', cipher_stack[0].name()
+            self.assertRaises(IndexError, cipher_stack.__getitem__, 2)
+            # For some reason there are 2 entries in the stack
+            #assert len(cipher_stack) == 1, len(cipher_stack)
+            assert s.get_cipher_list() == 'EXP-RC4-MD5', s.get_cipher_list()
+            
+            # Test Cipher_Stack iterator
+            i = 0
+            for cipher in cipher_stack:
+                i += 1
+                assert cipher.name() == 'EXP-RC4-MD5', '"%s"' % cipher.name()
+            # For some reason there are 2 entries in the stack
+            #assert i == 1, i
+            
             s.close()
         finally:
             self.stop_server(pid)
@@ -332,7 +356,8 @@ class SSLClientTestCase(unittest.TestCase):
         try:
             from M2Crypto import X509
             assert not ok
-            assert err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
+            assert err == m2.X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT or \
+                   err == m2.X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY or \
                    err == m2.X509_V_ERR_CERT_UNTRUSTED or \
                    err == m2.X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE
             assert m2.ssl_ctx_get_cert_store(ctx_ptr)
@@ -360,7 +385,211 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(data, 's_server -quiet -www') == -1)
 
+    def test_verify_allow_unknown_old(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9,
+                           SSL.cb.ssl_verify_callback)
+            ctx.set_allow_unknown_ca(1)
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_verify_allow_unknown_new(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9,
+                           SSL.cb.ssl_verify_callback_allow_unknown_ca)
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_verify_cert(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('ca.pem')
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_verify_cert_fail(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('server.pem')
+            s = SSL.Connection(ctx)
+            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            s.close()
+        finally:
+            self.stop_server(pid)
+
+    def test_verify_cert_mutual_auth(self):
+        self.args.extend(['-Verify', '2', '-CAfile', 'ca.pem'])        
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('ca.pem')
+            ctx.load_cert('x509.pem')
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_verify_cert_mutual_auth_servernbio(self):
+        self.args.extend(['-Verify', '2', '-CAfile', 'ca.pem', '-nbio'])
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('ca.pem')
+            ctx.load_cert('x509.pem')
+            s = SSL.Connection(ctx)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_verify_cert_mutual_auth_fail(self):
+        self.args.extend(['-Verify', '2', '-CAfile', 'ca.pem'])        
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('ca.pem')
+            s = SSL.Connection(ctx)
+            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            s.close()
+        finally:
+            self.stop_server(pid)
+
+    def test_verify_nocert_fail(self):
+        self.args.extend(['-nocert'])        
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            ctx.set_verify(SSL.verify_peer | SSL.verify_fail_if_no_peer_cert, 9)
+            ctx.load_verify_locations('ca.pem')
+            s = SSL.Connection(ctx)
+            self.assertRaises(SSL.SSLError, s.connect, self.srv_addr)
+            s.close()
+        finally:
+            self.stop_server(pid)
+
+    def test_HTTPSConnection(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            c = httpslib.HTTPSConnection(srv_host, srv_port)
+            c.request('GET', '/')
+            data = c.getresponse().read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_HTTPS(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import httpslib
+            c = httpslib.HTTPS(srv_host, srv_port)
+            c.putrequest('GET', '/')
+            c.putheader('Accept', 'text/html')
+            c.putheader('Accept', 'text/plain')
+            c.endheaders()
+            err, msg, headers = c.getreply()
+            assert err == 200, err
+            f = c.getfile()
+            data = f.read()
+            c.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_urllib(self):
+        pid = self.start_server(self.args)
+        try:
+            from M2Crypto import m2urllib
+            url = m2urllib.FancyURLopener()
+            url.addheader('Connection', 'close')
+            u = url.open('https://%s:%s/' % (srv_host, srv_port))
+            data = u.read()
+            u.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
+    def test_blocking0(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            s.setblocking(0)
+            self.assertRaises(Exception, s.connect, self.srv_addr)
+            s.close()
+        finally:
+            self.stop_server(pid)
+
+    def test_blocking1(self):
+        pid = self.start_server(self.args)
+        try:
+            ctx = SSL.Context()
+            s = SSL.Connection(ctx)
+            s.setblocking(1)
+            try:
+                s.connect(self.srv_addr)
+            except SSL.SSLError, e:
+                assert 0, e
+            data = self.http_get(s)
+            s.close()
+        finally:
+            self.stop_server(pid)
+        self.failIf(string.find(data, 's_server -quiet -www') == -1)
+
     def test_twisted_wrapper(self):
+        #
+        # LEAK ALERT!
+        # If this method is commented out, the leak detection code does
+        # not find any leaks.
+        #
+        
         # Test only when twisted and ZopeInterfaces are present
         try:
             from twisted.internet.protocol import ClientFactory
@@ -406,12 +635,16 @@ class SSLClientTestCase(unittest.TestCase):
             self.stop_server(pid)
         self.failIf(string.find(twisted_data, 's_server -quiet -www') == -1)
 
+
+twisted_data = ''
+
+class CheckerTestCase(unittest.TestCase):
     def test_checker(self):
         from M2Crypto.SSL import Checker
         from M2Crypto import X509
 
         check = Checker.Checker(host=srv_host,
-                                peerCertHash='545C873E4FF7E31BF439D80C9F5F128DF7E3EE57')
+                                peerCertHash='9594D272A975F58F4430511D15B4B7FF3D778113')
         x509 = X509.load_cert('server.pem')
         assert check(x509, srv_host)
         self.assertRaises(Checker.WrongHost, check, x509, 'example.com')
@@ -419,14 +652,18 @@ class SSLClientTestCase(unittest.TestCase):
         import doctest
         doctest.testmod(Checker)
         
+    
+class ContextTestCase(unittest.TestCase):
     def test_ctx_load_verify_locations(self):
         ctx = SSL.Context()
         self.assertRaises(AssertionError, ctx.load_verify_locations, None, None)
 
-twisted_data = ''
-
 def suite():
-    return unittest.makeSuite(SSLClientTestCase)
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(CheckerTestCase))
+    suite.addTest(unittest.makeSuite(ContextTestCase))
+    suite.addTest(unittest.makeSuite(SSLClientTestCase))
+    return suite    
     
 
 def zap_servers():
@@ -448,6 +685,13 @@ def zap_servers():
 
 
 if __name__ == '__main__':
+    report_leaks = 0
+    
+    if report_leaks:
+        import gc
+        gc.enable()
+        gc.set_debug(gc.DEBUG_LEAK & ~gc.DEBUG_SAVEALL)
+    
     try:
         Rand.load_file('../randpool.dat', -1) 
         unittest.TextTestRunner().run(suite())
@@ -455,25 +699,6 @@ if __name__ == '__main__':
     finally:
         zap_servers()
 
-
-"""
-Client tests:
-- server -verify
-- server -Verify
-- server -nbio
-- server -nbio_test
-- server -nocert
-+ simple server test
-+ server -cipher
-+ server -tls1
-+ server -no_tls1
-
-Server tests:
-- ???
-
-Others:
-- ssl_dispatcher
-- SSLServer
-- ForkingSSLServer
-- ThreadingSSLServer
-"""
+    if report_leaks:
+        import alltests
+        alltests.dump_garbage()
